main() {
    if (rdp_session_found()) {
        println("WARNING:")
        println("")
        if (active_rdp_session_found()) {
            println("  Active RDP (Remote Desktop Protocol/Client) session")
            println("  found.")
            println("")
            println("  Disconnecting or minimizing the RDP client causes")
            println("  lack of a display.")
            println("")
            println("  Computer connected via RDP: " . get_rdp_client_name())
        } else {
            println("  Disconnected RDP (Remote Desktop Protocol/Client)")
            println("  session found.")
            println("")
            println("  This is usually caused by connecting and then")
            println("  disconnecting (or minimizing) an RDP client to a")
            println("  (Windows) computer. Once disconnected or minimized")
            println("  the user session then lacks a display.")
        }
        println("")
        println("")
        println("")
        println("  Lack of a display causes these general problems for")
        println("  any GUI automation on Windows:")
        println("")
        println("  1) Applications cannot render their GUI.")
        println("")
        println("     Without rendering the coordinates of GUI elements")
        println("     on the screen cannot be determined. These")
        println("     coordinates are required for many mouse based")
        println("     interactions with applications.")
        println("")
        println("  2) Sending ""Window Messages"" is paused or hangs.")
        println("")
        println("     Among others, sending Window Messages is required")
        println("     for querying the current GUI controls of")
        println("     applications.")
        println("")
        println("")
        println("")
        println("  To restore the display, log in to the physical console")
        println("  (or to a VNC server running as a Windows service, if")
        println("  one is installed).")
        println("")
    }

    if (screensaver_is_active()) {
        println("Display not available (Reason: Screensaver is active)")
        exitapp 1
    }

    if (workstation_is_locked()) {
        println("Display not available (Reason: Workstation is locked)")
        exitapp 2
    }

    if (!display_available()) {
        println("Display not available (Reasons: (Typically) Disconnected RDP session or minimized RDP client.)")
        exitapp 3
    }

    println("Display available")
    exitapp 0
}


active_rdp_session_found() {
    if (!rdp_session_found()) {
        return false
    }

    ; The RDP session is only active when a client
    ; name can be determined:
    if (strlen(get_rdp_client_name()) == 0) {
        return false
    }

    return true
}


disconnected_rdp_session_found() {
    if (!rdp_session_found()) {
        return false
    }

    ; The RDP session is only active when a client
    ; name can be determined:
    if (strlen(get_rdp_client_name()) == 0) {
        return false
    }

    return true
}


rdp_session_found() {
    SysGet v, 4096
    if (v == "0") {
        return false
    }
    return true
}


screensaver_is_active() {
	SPI_GETSCREENSAVERRUNNING = 0x0072
	result := DllCall( "user32.dll\SystemParametersInfo", "uint", SPI_GETSCREENSAVERRUNNING, "uint", 0, "uint*", screen_saver_active, "uint", 0 )
	if (screen_saver_active > 0)
	{
        return true
    }
    return false
}


workstation_is_locked() {
    ; https://autohotkey.com/boards/viewtopic.php?t=8023
	return !DllCall("User32\OpenInputDesktop","int",0*0,"int",0*0,"int",0x0001L*1)
}


; To actually see this output to stdout the output must
; be redirected or piped to another command:
;
;  display_available | more
println(text) {
    fileappend % text, *
    fileappend % "`r`n", *
}


display_available() {
    ; winexist("a") returns "0x0" without a display:
    if (winexist("a") == "0x0")
    {
        return false
    }
    return true
}


get_rdp_client_name() {
    Process Exist
    if !DllCall("ProcessIdToSessionId", "uint", ErrorLevel, "uint*", sid)
    {
        return ""
    }
    ; Defined in Windows SDK - WtsApi32.h
    WTS_CURRENT_SERVER_HANDLE   :=  0
    WTSClientName               := 10
    WTSClientAddress            := 14

    ; WTSQuerySessionInformation:
    ;   http://msdn.microsoft.com/en-us/library/windows/desktop/aa383838

    /*
     *  Get client name.
     */

    if !DllCall("wtsapi32\WTSQuerySessionInformation"
                , "ptr", WTS_CURRENT_SERVER_HANDLE
                , "uint", sid, "int", WTSClientName
                , "ptr*", buf, "uint*", buf_size)
    {
        return ""
    }

    name := StrGet(buf+0)
    DllCall("wtsapi32\WTSFreeMemory", "ptr", buf)

    if !name
    {
        return ""
    }
    return name
}


main()
